divert(-1)

# "coserv1.m4"	K. J. Turner (kjt@cs.stir.ac.uk)	04/06/96
# 
# Copyright 1996 K. J. Turner, University of Stirling

# This "m4" macro file uses SAGE (Service Attribute Generator) to
# generate a LOTOS specification of a connection-oriented service. Its
# characteristics are as follows:
# 
#   o There is just a pair of users, 1 and 2
#   
#   o User 1 may initiate a connection. This is user-confirmed, reliable and
#   has primitives:
#   
#     ConnReq (Addr1, Addr2)
#     ConnInd (Addr1, Addr2)
#     ConnRes ()
#     ConnCon ()
#     
#   A connection initiation attempt may be withheld (e.g. due to lack of
#   resources). Completion of connection enables data transfer: normal data,
#   expedited data, reset. Connections may be established for any pair of
#   interaction group identifiers.
#     
#   o Either user may initiate normal data transfer. This is
#   provider-confirmed, unreliable and has primitives:
#   
#     DataReq (Data)
#     DataInd (Data)
#     AckInd ()
#     
#   o User 2 may initiate expedited data transfer. This is unconfirmed,
#   reliable and has primitives:
#   
#     ExpReq (Data)
#     ExpInd (Data)
#     
#   Expedited data from user 2 may overtake normal data from user 2. Normal
#   and expedited data in either direction is interleaved.
#     
#   o User 1 may initiate reset. This is unconfirmed, reliable and has
#   primitives:
#   
#     ResetReq (Reas)
#     ResetInd (Reas)
#     
#   Simultaneous resets by both users acknowledge each other. Reset
#   interrupts then resumes data transfer.
#     
#   o User 2 may initiate disconnection. This is unconfirmed, occurs strictly
#   consecutively and has primitives:
#   
#     DiscReq ()
#     DiscInd ()
#     
#   Disconnection breaks any existing connection then allows a further
#   connection attempt.

include(sage.m4)

divert

define(Conn12,facility(12,user_confirmed,reliable,Conn(Addr1,Addr2),Conn()))
define(Data12,facility(12,provider_confirmed,unreliable,Data(Data),Ack()))
define(Data21,facility(21,provider_confirmed,unreliable,Data(Data),Ack()))
define(Exp21,facility(21,unconfirmed,reliable,Exp(Data)))
define(Reset12,facility(12,unconfirmed,reliable,Reset(Reas)))
define(Disc21,facility(21,unconfirmed,consecutive,Disc()))

global(cs,
  withheld(Conn,
    forall_ids(
      disables(Disc21,
	enables(Conn12,
	  interrupts(
	    colliding(Reset12),
	      interleaves(Data12,
	        overtakes(Exp21,Data21))))))))
